@@ -216,6 +216,13 @@ class HttpParser:
             self.has_header(b'Connection') and \
             self.has_header(b'Upgrade')
 
+    @property
+    def is_websocket_upgrade(self) -> bool:
+        return self.is_connection_upgrade and self.header(b'upgrade').lower() in (
+            b'websocket',
+            b'derp',
+        )
+
     @property
     def is_https_tunnel(self) -> bool:
         """Returns true for HTTPS CONNECT tunnel request."""
@@ -357,7 +364,7 @@ class HttpParser:
         #
         # See TestHttpParser.test_issue_398 scenario
         self.state = httpParserStates.RCVING_BODY
-        self.body = raw
+        self.body = bytes(raw)
         return False, memoryview(b'')
 
     def _process_headers(self, raw: memoryview) -> Tuple[bool, memoryview]:
@@ -417,7 +424,9 @@ class HttpParser:
                     break
                 # To avoid a possible attack vector, we raise exception
                 # if parser receives an invalid request line.
-                raise HttpProtocolException('Invalid request line %r' % raw)
+                raise HttpProtocolException(
+                    'Invalid request line %r' % raw.tobytes(),
+                )
             parts = line.split(WHITESPACE, 2)
             self.version = parts[0]
             self.code = parts[1]
